package provide Colors 1.0

##  colors.tcl
##
##	colors : utilities for color processing
##             RGB <--> HSL 
##
##
## This library is free software; you can use, modify, and redistribute it
## for any purpose, provided that existing copyright notices are retained
## in all copies and that this notice is included verbatim in any
## distributions.
##

 

namespace eval Colors {;}
 

     #   red     -->  #ff0000
     #   #112233 -->  #112233
   proc Colors::TkColor2RRGGBB {color} {      
       set res "#"
       foreach x [winfo rgb . $color] { 
           append res [format "%02x" [expr $x/256]] 
       }
       return $res
   }
 
     #   #RRGGBB --> { r g b }   (r,g,b are 0..255 integers)
   proc Colors::RRGGBB2R_G_B {rrggbb} {
       scan $rrggbb "#%2x%2x%2x" r g b
       return [list $r $g $b]
   }
  
   proc Colors::RgbList2RRGGBB {rgb} {
       set res "#"
       foreach x $rgb { 
           append res [format "%02x" $x] 
       }
       return $res
   }
  


  # General info
  #
  #  RGB <--> HSL
  #   r,g,b  are integers  0..255
  #
  #   h (hue)  is a 0.0..360.0 angle
  #   s (saturation)  is 0.0 .. 1.0
  #   l (luminance)   is 0.0 .. 1.0  ( 0 is black, 1 is white )
  #
  #  RGB <--> HSV 
  #    r,g,b  are integers 0..65535
  #    h  0.0..1.0
  #    s  0.0..1.0
  #    v  0.0..1.0
  #    NOTE: these latters transformation are not widely tested ..
  #          It could be useful to harmonize all parameters
 

# --- HSV - RGB --------------------------------------------------

 # credits:
 #   http://en.wikipedia.org/wiki/HLS_color_space

proc Colors::HslToRgb { H S L } {
    if { $L < 0.5 } {
       set Q [expr $L*(1.0+$S)]
    } else {
       set Q [expr $L+$S-($L*$S)]
    }
    set P [expr 2.0*$L-$Q]
    set Hk [expr $H/360.0]
    set T(r) [expr $Hk+1.0/3.0]
    set T(g) $Hk
    set T(b) [expr $Hk-1.0/3.0]

     # normalize
    foreach c {r g b} {
       if {$T($c) < 0.0} { set T($c) [expr $T($c)+1.0] }
       if {$T($c) > 1.0} { set T($c) [expr $T($c)-1.0] }
    }

    foreach c {r g b} {
       if {$T($c) < [expr 1.0/6.0]} {
         set T($c) [expr $P+($Q-$P)*6.0*$T($c)]
       } elseif {$T($c) < 0.5} {
         set T($c) $Q
       } elseif {$T($c) < [expr 2.0/3.0]} {
         set T($c) [expr $P+($Q-$P)*(2.0/3.0-$T($c))*6.0]
       } else {
         set T($c) $P
       }
       set T($c) [expr round($T($c)*255)]
    }

    return [list $T(r) $T(g) $T(b)]
}


 #   Test:
 #     RgbToHsl 255 0   0    --> 0      1    0.5
 #     RgbToHsl 127 255 127  --> 120.0  1.0  0.749019607843
 #     RgbToHsl 128 255 128  --> 120.0  1.0  0.750980392157
 #     RgbToHsl 0   0   127  --> 240.0  1.0  0.249019607843
 #     RgbToHsl 0   0   127  --> 240.0  1.0  0.250980392157

proc Colors::RgbToHsl { r g b } {
   set r [expr $r/255.0]
   set g [expr $g/255.0]   
   set b [expr $b/255.0]

   set max $r
   set min $r
   if { $g > $max } { set max $g }
   if { $g < $min } { set min $g }
   if { $b > $max } { set max $b }
   if { $b < $min } { set min $b }

   if { $max == $min } {
     set H 0.0
   } elseif { $b == $max } {
      set H [expr 60* ($r-$g)/($max-$min)+240]
   } elseif { $g == $max } {
      set H [expr 60* ($b-$r)/($max-$min)+120]
   } else {
      # $r == $max
     if { $g >= $b } {
         set H [expr 60* ($g-$b)/($max-$min)]
     } else {
         set H [expr 60* ($g-$b)/($max-$min)+360]
     }
   }

   set L [expr ($max+$min)/2]
   
   if { $L == 0.0  ||  $max == $min } {
     set S 0.0
   } elseif { $L <= 0.5 } {
     set S [expr ($max-$min)/($max+$min)]
   } else {
     set S [expr ($max-$min)/(2.0-($max+$min))]
   }

   return [list $H $S $L]
}
 

# --- HSV - RGB --------------------------------------------------

# --
#  Taken from tk8.0/demos/tcolor.tcl
# --
# The procedure below converts an HSB value to RGB.  It takes hue, saturation,
# and value components (floating-point, 0-1.0) as arguments, and returns a
# list containing RGB components (integers, 0-65535) as result.  The code
# here is a copy of the code on page 616 of "Fundamentals of Interactive
# Computer Graphics" by Foley and Van Dam.

proc Colors::HsvToRgb {hue sat val} {
    set v [expr {round(65535.0*$val)}]
    if {$sat == 0} {
	return [list $v $v $v]
    } else {
	set hue [expr {$hue*6.0}]
	if {$hue >= 6.0} {
	    set hue 0.0
	}
	set i [expr {int($hue)}]
	set f [expr {$hue-$i}]
	set p [expr {round(65535.0*$val*(1 - $sat))}]
        set q [expr {round(65535.0*$val*(1 - ($sat*$f)))}]
        set t [expr {round(65535.0*$val*(1 - ($sat*(1 - $f))))}]
        switch $i {
	    0 {return [list $v $t $p]}
	    1 {return [list $q $v $p]}
	    2 {return [list $p $v $t]}
	    3 {return [list $p $q $v]}
	    4 {return [list $t $p $v]}
            5 {return [list $v $p $q]}
        }
    }
}


# --
#  Taken from tk8.0/demos/tcolor.tcl
# --
# The procedure below converts an RGB value to HSB.  It takes red, green,
# and blue components (0-65535) as arguments, and returns a list containing
# HSB components (floating-point, 0-1) as result.  The code here is a copy
# of the code on page 615 of "Fundamentals of Interactive Computer Graphics"
# by Foley and Van Dam.

proc Colors::RgbToHsv {red green blue} {
    if {$red > $green} {
	set max $red.0
	set min $green.0
    } else {
	set max $green.0
	set min $red.0
    }
    if {$blue > $max} {
	set max $blue.0
    } else {
	if {$blue < $min} {
	    set min $blue.0
	}
    }
    set range [expr {$max-$min}]
    if {$max == 0} {
	set sat 0
    } else {
	set sat [expr {($max-$min)/$max}]
    }
    if {$sat == 0} {
	set hue 0
    } else {
	set rc [expr {($max - $red)/$range}]
	set gc [expr {($max - $green)/$range}]
	set bc [expr {($max - $blue)/$range}]
	if {$red == $max} {
	    set hue [expr {.166667*($bc - $gc)}]
	} else {
	    if {$green == $max} {
		set hue [expr {.166667*(2 + $rc - $bc)}]
	    } else {
		set hue [expr {.166667*(4 + $gc - $rc)}]
	    }
	}
	if {$hue < 0.0} {
	    set hue [expr {$hue + 1.0}]
	}
    }
    return [list $hue $sat [expr {$max/65535}]]
}
